
# q-expansion of newform 399.2.j.e, downloaded from the LMFDB on 15 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 399
weight = 2
poly_data = [16, -8, 8, -8, 1, -4, 2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-8, 8, 6, 7, 0, 0, -1, -1], 8], [[8, -8, 8, -1, 4, -2, 1, -1], 8], [[16, 2, 7, -1, 0, -3, 0, -1], 4], [[-16, -8, -10, 1, 0, 4, 1, 1], 4], [[-32, -4, -20, 3, 0, 6, 1, 3], 8], [[-8, 1, -5, 1, -1, 2, 0, 1], 2]]

hecke_ring_character_values = [[134, [1, 0, 0, 0, 0, 0, 0, 0]], [115, [-1, 0, 0, 0, 1, 0, 0, 0]], [211, [1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 1, 0, 1, 0, 0], [0, 0, 0, 0, -1, 0, 0, 0], [-3, 0, -1, 1, 2, 1, -1, 2], [1, -2, 1, 0, 1, -1, 0, 0], [0, -1, 0, -2, 1, 2, 3, 0], [-1, 3, 0, 3, 0, 1, -1, 0], [0, 0, 0, 0, -3, 0, 0, 0], [1, 0, 0, 0, -1, 0, 0, 0], [2, -3, 0, 0, -2, -3, -3, 0], [-4, -2, 1, -1, 0, 1, -2, 1], [0, 1, 0, -1, -6, 1, 0, 0], [2, -1, 1, 0, -1, -1, 0, -2], [-1, 1, 0, 1, 0, -2, 2, 0], [1, 3, -2, 1, 0, 0, 2, -2], [-3, 2, 0, 2, 3, 4, 2, 0], [0, -1, 0, 2, 3, -2, -1, 0], [0, 3, 0, 1, 2, -1, -4, 0], [0, 1, 1, -1, 1, 0, 2, -2], [0, -5, 0, 5, -3, -5, 0, 0], [-4, 3, -2, 1, 0, 3, -1, -2], [0, -3, 0, -1, 3, 1, 4, 0], [6, -1, 3, -4, -3, -5, 2, -6], [-5, 1, 1, 2, 0, 4, -5, 1], [2, -1, 1, -4, -1, -5, 0, -2], [1, -3, 1, -2, 0, -1, 0, 1], [0, 3, 0, 1, 3, -1, -4, 0], [-11, 3, -1, 0, 10, 3, 2, 2], [-7, -3, -2, 6, 5, 3, -5, 4], [2, -1, 2, -7, 3, 5, 6, -1], [0, -3, 1, -2, 0, -4, 3, 1], [-3, 2, -3, -1, 0, 5, -2, -3], [5, -2, 3, 2, -2, 0, 1, -6], [-4, 0, -4, 6, 2, -2, -2, 2], [2, -6, 4, -2, 0, -5, 1, 4], [6, 2, 0, 2, -6, 4, 2, 0], [-4, 5, -4, 1, -13, 3, -2, 2], [-4, 1, -4, -3, 5, 7, 6, 2], [-1, 8, 0, 0, 1, 8, 8, 0], [-5, -1, 0, -1, 0, 3, -3, 0], [3, -3, 5, -1, 2, -4, 2, -10], [4, -9, 4, 2, 9, -6, 3, -2], [-4, -1, 1, 0, 0, 6, -7, 1], [-12, -7, -3, 8, 9, 1, -10, 6], [-2, 4, -2, 2, 8, 0, -4, 1], [0, 0, -1, -1, 0, -7, 8, -1], [-6, 1, -6, 2, -5, 4, 3, 3], [-1, 1, -1, 0, 0, -3, 4, -1], [18, 1, 0, 1, 0, 0, 0, 0], [-6, 4, -6, -1, -4, 7, 3, 3], [-10, 5, 1, 4, 11, 9, 6, -2], [-9, 1, -6, 8, 3, 9, -5, 12], [0, -2, 1, -1, 0, 4, -5, 1], [6, -7, 6, 2, -12, -8, -1, -3], [-6, -1, -2, -3, 0, -3, 5, -2], [-7, 5, -4, -3, 3, 2, 1, 8], [0, 3, 0, -2, -16, 2, -1, 0], [-2, -1, -2, 4, -7, -2, -1, 1], [-18, -1, -8, 11, 10, 10, -9, 16], [-2, 2, -2, 3, 20, -1, -3, 1], [-4, -3, 2, -1, 0, -5, 3, 2], [2, 0, 2, -3, 7, 1, 1, -1], [-3, -1, 2, 1, 0, 3, -5, 2], [5, -4, -1, -5, 0, 0, 1, -1], [6, -7, 6, 6, -11, -12, -5, -3], [3, -5, 2, 0, -1, -5, -3, -4], [8, 1, -3, 6, -11, 7, -2, 6], [7, -1, 4, 0, -3, -1, 3, -8], [2, 13, 0, 13, 0, 8, -8, 0], [4, 4, 4, -3, -4, -1, -5, -2], [-1, 8, 1, 9, 0, -1, 0, 1], [2, 3, 2, -4, -14, 2, -1, -1], [19, 1, 2, 0, -17, 1, 3, -4], [0, 11, 0, -15, -6, 15, 4, 0], [3, -8, 6, 3, 3, -5, -2, -12], [14, -17, 4, -13, 0, -10, 6, 4], [-10, -11, -3, 5, 7, -6, -14, 6], [10, -6, 10, -2, -3, -8, -2, -5], [-15, 9, -2, -1, 13, 8, 7, 4], [8, 4, 3, -1, -5, 3, 7, -6], [-10, 3, -10, 1, 6, 9, 6, 5], [-6, 3, 3, 6, 0, -4, 1, 3], [0, -3, 2, -1, 0, 0, -2, 2], [2, -2, 2, -1, -14, -1, 1, -1], [11, 1, 2, 3, 0, -4, 2, 2], [-13, 11, 0, -1, 13, 10, 11, 0], [2, -4, -3, 1, -5, -3, -7, 6], [20, -16, 1, -15, 0, -7, 6, 1], [-17, 0, -2, -5, 15, -5, -2, 4], [3, 7, -7, 0, 0, 0, 7, -7], [10, -3, -4, -7, 0, 5, -1, -4], [18, 3, 0, -8, -18, -5, 3, 0], [8, -12, 8, -3, 0, -5, 7, -4], [-2, -6, -2, 17, 9, -15, -9, 1], [3, 16, -3, 13, 0, 9, -6, -3], [6, -4, -2, 15, -8, 11, -6, 4], [18, -12, 8, -4, 0, -5, -3, 8], [22, 1, 1, 1, -21, 2, 2, -2], [6, -1, 6, -6, 0, 0, 1, -3], [-5, 1, -7, 16, -2, 17, -6, 14], [8, -2, 1, -7, -7, -9, -1, -2], [-7, 2, 0, 2, 0, 15, -15, 0], [-14, 12, -14, -11, 27, 25, 13, 7], [-2, -2, -2, -5, 29, 7, 9, 1], [6, -1, 2, -7, -4, -8, 1, -4], [6, -12, 6, -1, -10, -5, 7, -3], [6, -8, 6, -10, 2, 4, 12, -3], [8, 4, -3, 1, 0, 5, -2, -3], [-7, 15, -5, -4, 2, 11, 10, 10], [4, -10, 4, 2, 25, -6, 4, -2], [21, -6, -4, -10, 0, 5, -1, -4], [-12, -5, -5, 13, 7, 8, -10, 10], [2, -4, 2, -4, -10, 2, 6, -1], [-12, 15, -3, 12, 0, 11, -8, -3], [2, -7, 2, 11, 3, -13, -6, -1], [11, 2, 0, 2, 0, -6, 6, 0], [0, 13, 0, 1, -5, -1, -14, 0], [4, 1, -6, -5, 0, 6, 0, -6], [-14, -2, -14, 8, -3, 6, 8, 7], [-21, 11, 0, 4, 21, 15, 11, 0], [-1, -2, 4, 2, 0, -6, 2, 4], [0, 9, 0, -10, 13, 10, 1, 0], [-13, 8, 2, 10, 0, -1, -1, 2], [7, 5, 0, -13, -7, -8, 5, 0], [6, -10, 6, -3, -4, -3, 7, -3], [-7, -13, -3, 6, 4, -7, -16, 6], [-14, 3, -4, -1, 0, 9, -5, -4], [-19, 10, -3, -10, 16, 0, 7, 6], [-15, -5, 2, -4, 17, -9, -3, -4], [-4, -11, 2, -9, 0, -15, 13, 2], [26, -3, 11, -12, -15, -15, 8, -22], [-10, 2, -10, -5, 6, 15, 13, 5], [-2, -4, -4, -8, 0, 1, 3, -4], [1, 0, 3, -1, 2, -1, 3, -6], [17, 1, -10, -9, 0, -3, 13, -10], [3, -3, -4, 11, -7, 8, -7, 8], [6, -15, -3, -18, 0, -12, 15, -3], [0, -7, 0, -4, -18, 4, 11, 0], [12, -7, 12, 8, -11, -20, -13, -6], [-10, 6, -2, 4, 0, 0, 2, -2], [0, 9, 0, -1, 14, 1, -8, 0], [-4, 12, -1, 11, 0, 13, -12, -1], [3, 1, 6, -8, 3, -7, 7, -12], [-12, 17, -12, 7, -9, 5, -12, 6], [-9, -4, 3, -1, 0, 3, -6, 3], [-2, -6, -2, 8, 3, -6, 0, 1], [10, -12, 0, -12, 0, -4, 4, 0], [11, 8, -3, 5, 0, -9, 12, -3], [-8, 6, -8, 16, 18, -8, -14, 4], [-4, -11, 1, 14, 5, 3, -10, -2], [31, 9, 10, -12, -21, -3, 19, -20], [12, -6, 9, -4, -3, -10, 3, -18], [15, -15, -4, -19, 0, -10, 14, -4], [-14, -6, -1, -7, 0, -3, 4, -1], [3, -5, 10, -3, 7, -8, 5, -20], [0, 21, 0, -8, 4, 8, -13, 0], [-13, 12, 0, 12, 0, 15, -15, 0], [-2, 11, -5, -4, -3, 7, 6, 10], [8, 9, -1, 0, -9, 9, 8, 2], [-8, -5, 7, 2, 0, 2, -9, 7], [-2, -6, -2, -9, 2, 11, 17, 1], [17, -7, 0, 11, -17, 4, -7, 0], [-6, 9, -5, 4, 0, 16, -11, -5], [3, 31, -7, 24, 0, 7, 0, -7], [-14, -4, 2, 6, 16, 2, -2, -4], [2, -17, 2, -9, -9, 7, 24, -1], [14, -21, 14, 18, -20, -32, -11, -7], [6, 5, 6, 13, -27, -19, -24, -3], [-24, 7, -24, 3, 20, 21, 14, 12]]
